"""
插件瀏覽器 Widget / Plugin Browser Widget

提供 UI 讓使用者瀏覽 GitHub Repo 上的插件，檢視資訊並下載到 jeditor_plugins/。
Provides a UI for browsing plugins on a GitHub repo, viewing info, and downloading to jeditor_plugins/.
"""
from __future__ import annotations

from pathlib import Path
from threading import Thread

from PySide6.QtCore import Qt, Signal, QObject
from PySide6.QtGui import QFont
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLineEdit, QPushButton,
    QTreeWidget, QTreeWidgetItem, QLabel, QTextEdit, QMessageBox,
    QHeaderView, QSplitter, QProgressBar,
)

from je_editor.pyside_ui.main_ui.plugin_browser.github_api import (
    fetch_repo_tree, fetch_plugin_metadata, download_plugin_file,
)
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

# 預設插件 Repo / Default plugin repo
_DEFAULT_REPO_URL = "https://github.com/Jeffrey-Plugin-Repos/IDE_Plugins"


class _WorkerSignals(QObject):
    """用於背景執行緒回報結果的信號 / Signals for background thread results."""
    finished = Signal(list)       # 插件列表 / Plugin list
    metadata = Signal(dict)       # 插件元資料 / Plugin metadata
    error = Signal(str)           # 錯誤訊息 / Error message
    download_done = Signal(str)   # 下載完成路徑 / Download done path


class PluginBrowserWidget(QWidget):
    """
    插件瀏覽器主 Widget。
    Plugin browser main widget.
    """

    def __init__(self, parent=None):
        super().__init__(parent)
        jeditor_logger.info("Init PluginBrowserWidget")

        self._signals = _WorkerSignals()
        self._signals.finished.connect(self._on_plugins_loaded)
        self._signals.metadata.connect(self._on_metadata_loaded)
        self._signals.error.connect(self._on_error)
        self._signals.download_done.connect(self._on_download_done)

        self._plugins: list[dict] = []
        self._current_plugin: dict | None = None

        self._init_ui()

    def _init_ui(self):
        main_layout = QVBoxLayout(self)

        # === 頂部：Repo URL 輸入 ===
        # === Top: Repo URL input ===
        top_layout = QHBoxLayout()
        self._repo_label = QLabel(
            language_wrapper.language_word_dict.get("plugin_browser_repo_label", "Repository URL:"))
        self._repo_input = QLineEdit(_DEFAULT_REPO_URL)
        self._repo_input.setPlaceholderText("https://github.com/owner/repo")
        self._fetch_btn = QPushButton(
            language_wrapper.language_word_dict.get("plugin_browser_fetch_btn", "Fetch Plugins"))
        self._fetch_btn.clicked.connect(self._on_fetch_clicked)

        top_layout.addWidget(self._repo_label)
        top_layout.addWidget(self._repo_input, stretch=1)
        top_layout.addWidget(self._fetch_btn)
        main_layout.addLayout(top_layout)

        # === 進度條 ===
        # === Progress bar ===
        self._progress = QProgressBar()
        self._progress.setRange(0, 0)  # indeterminate
        self._progress.setVisible(False)
        main_layout.addWidget(self._progress)

        # === 中間：插件列表 + 詳細資訊 ===
        # === Middle: plugin list + detail ===
        splitter = QSplitter(Qt.Orientation.Horizontal)

        # 左側：插件樹狀清單 / Left: plugin tree
        self._tree = QTreeWidget()
        self._tree.setHeaderLabels([
            language_wrapper.language_word_dict.get("plugin_browser_col_name", "Plugin"),
            language_wrapper.language_word_dict.get("plugin_browser_col_path", "Path"),
            language_wrapper.language_word_dict.get("plugin_browser_col_size", "Size"),
        ])
        self._tree.header().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
        self._tree.header().setSectionResizeMode(1, QHeaderView.ResizeMode.ResizeToContents)
        self._tree.header().setSectionResizeMode(2, QHeaderView.ResizeMode.ResizeToContents)
        self._tree.currentItemChanged.connect(self._on_item_selected)
        splitter.addWidget(self._tree)

        # 右側：詳細資訊面板 / Right: detail panel
        detail_widget = QWidget()
        detail_layout = QVBoxLayout(detail_widget)
        detail_layout.setContentsMargins(4, 4, 4, 4)

        self._detail_title = QLabel()
        title_font = QFont()
        title_font.setPointSize(14)
        title_font.setBold(True)
        self._detail_title.setFont(title_font)
        detail_layout.addWidget(self._detail_title)

        self._detail_info = QLabel()
        self._detail_info.setWordWrap(True)
        detail_layout.addWidget(self._detail_info)

        self._detail_text = QTextEdit()
        self._detail_text.setReadOnly(True)
        self._detail_text.setPlaceholderText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_select_hint", "Select a plugin to view details"))
        detail_layout.addWidget(self._detail_text, stretch=1)

        # 下載按鈕 / Download button
        btn_layout = QHBoxLayout()
        self._download_btn = QPushButton(
            language_wrapper.language_word_dict.get("plugin_browser_download_btn", "Download && Install"))
        self._download_btn.setEnabled(False)
        self._download_btn.clicked.connect(self._on_download_clicked)
        btn_layout.addStretch()
        btn_layout.addWidget(self._download_btn)
        detail_layout.addLayout(btn_layout)

        splitter.addWidget(detail_widget)
        splitter.setStretchFactor(0, 2)
        splitter.setStretchFactor(1, 3)
        main_layout.addWidget(splitter, stretch=1)

        # === 底部：狀態列 ===
        # === Bottom: status bar ===
        self._status = QLabel(
            language_wrapper.language_word_dict.get(
                "plugin_browser_status_ready", "Ready"))
        main_layout.addWidget(self._status)

    # ----- Actions / 動作 -----

    def _on_fetch_clicked(self):
        """使用者按下 Fetch 按鈕 / User clicks Fetch button."""
        repo_url = self._repo_input.text().strip().rstrip("/")
        # 解析 owner/repo / Parse owner/repo
        # 支援格式: https://github.com/owner/repo 或 owner/repo
        # Supports: https://github.com/owner/repo or owner/repo
        if "github.com/" in repo_url:
            parts = repo_url.split("github.com/")[1].split("/")
        else:
            parts = repo_url.split("/")

        if len(parts) < 2:
            self._on_error(language_wrapper.language_word_dict.get(
                "plugin_browser_invalid_url", "Invalid repository URL"))
            return

        owner, repo = parts[0], parts[1]
        self._set_loading(True)
        self._status.setText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_status_fetching", "Fetching plugin list..."))
        self._tree.clear()
        self._plugins.clear()

        Thread(target=self._fetch_worker, args=(owner, repo), daemon=True).start()

    def _fetch_worker(self, owner: str, repo: str):
        """背景取得插件列表 / Background fetch plugin list."""
        try:
            plugins = fetch_repo_tree(owner, repo)
            # 為每個插件取得元資料 / Fetch metadata for each plugin
            for plugin in plugins:
                url = plugin.get("download_url", "")
                if url:
                    meta = fetch_plugin_metadata(url)
                    plugin.update(meta)
            self._signals.finished.emit(plugins)
        except Exception as e:
            self._signals.error.emit(str(e))

    def _on_plugins_loaded(self, plugins: list[dict]):
        """插件列表載入完成 / Plugin list loaded."""
        self._set_loading(False)
        self._plugins = plugins

        # 按目錄分群 / Group by directory
        groups: dict[str, list[dict]] = {}
        for p in plugins:
            parts = p["path"].split("/")
            group = parts[0] if len(parts) > 1 else ""
            groups.setdefault(group, []).append(p)

        self._tree.clear()
        for group_name, group_plugins in sorted(groups.items()):
            if group_name:
                # 有目錄的分群 / Grouped under directory
                group_item = QTreeWidgetItem(self._tree, [group_name, "", ""])
                group_item.setExpanded(True)
                for p in sorted(group_plugins, key=lambda x: x["name"]):
                    display = p.get("PLUGIN_NAME", p["name"])
                    child = QTreeWidgetItem(group_item, [
                        display, p["path"], f'{p["size"]} B'
                    ])
                    child.setData(0, Qt.ItemDataRole.UserRole, p)
            else:
                # 根目錄的檔案 / Root-level files
                for p in sorted(group_plugins, key=lambda x: x["name"]):
                    display = p.get("PLUGIN_NAME", p["name"])
                    item = QTreeWidgetItem(self._tree, [
                        display, p["path"], f'{p["size"]} B'
                    ])
                    item.setData(0, Qt.ItemDataRole.UserRole, p)

        count = len(plugins)
        self._status.setText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_status_loaded", "Loaded {count} plugins").format(count=count))

    def _on_item_selected(self, current: QTreeWidgetItem, _previous):
        """使用者選取插件 / User selects a plugin."""
        if current is None:
            return
        plugin = current.data(0, Qt.ItemDataRole.UserRole)
        if plugin is None:
            # 群組節點 / Group node
            self._download_btn.setEnabled(False)
            self._detail_title.setText("")
            self._detail_info.setText("")
            self._detail_text.setPlainText("")
            return

        self._current_plugin = plugin
        self._download_btn.setEnabled(True)

        name = plugin.get("PLUGIN_NAME", plugin["name"])
        author = plugin.get("PLUGIN_AUTHOR", "")
        version = plugin.get("PLUGIN_VERSION", "")

        self._detail_title.setText(name)
        info_parts = []
        if version:
            info_parts.append(f"Version: {version}")
        if author:
            info_parts.append(f"Author: {author}")
        info_parts.append(f"File: {plugin['name']}")
        info_parts.append(f"Path: {plugin['path']}")
        self._detail_info.setText("\n".join(info_parts))

        # 顯示「載入原始碼…」並背景下載 / Show "loading source..." and fetch in background
        self._detail_text.setPlainText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_loading_source", "Loading source code..."))

        Thread(
            target=self._fetch_source_worker,
            args=(plugin.get("download_url", ""),),
            daemon=True,
        ).start()

    def _fetch_source_worker(self, url: str):
        """背景下載原始碼預覽 / Background fetch source preview."""
        try:
            from je_editor.pyside_ui.main_ui.plugin_browser.github_api import _download_text
            source = _download_text(url)
            self._signals.metadata.emit({"_source": source})
        except Exception as e:
            self._signals.metadata.emit({"_source": f"Error: {e}"})

    def _on_metadata_loaded(self, data: dict):
        """原始碼預覽載入完成 / Source preview loaded."""
        source = data.get("_source", "")
        self._detail_text.setPlainText(source)

    def _on_download_clicked(self):
        """使用者按下下載按鈕 / User clicks download button."""
        if self._current_plugin is None:
            return

        plugin = self._current_plugin
        download_url = plugin.get("download_url", "")
        file_name = plugin["name"]

        if not download_url:
            self._on_error("No download URL available")
            return

        # 直接下載到 jeditor_plugins/ 根目錄
        # Download directly to jeditor_plugins/ root
        dest_dir = self._get_plugins_dir()
        installed = dest_dir / file_name

        # 確認下載 / Confirm download
        if installed.exists():
            reply = QMessageBox.question(
                self,
                language_wrapper.language_word_dict.get(
                    "plugin_browser_overwrite_title", "Plugin exists"),
                language_wrapper.language_word_dict.get(
                    "plugin_browser_overwrite_msg",
                    "{name} already exists. Overwrite?").format(name=file_name),
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
            )
            if reply != QMessageBox.StandardButton.Yes:
                return

        self._set_loading(True)
        self._download_btn.setEnabled(False)
        self._status.setText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_status_downloading", "Downloading {name}...").format(name=file_name))

        Thread(
            target=self._download_worker,
            args=(download_url, str(dest_dir), file_name),
            daemon=True,
        ).start()

    def _download_worker(self, url: str, dest_dir: str, file_name: str):
        """背景下載插件 / Background download plugin."""
        try:
            saved = download_plugin_file(url, dest_dir, file_name)
            self._signals.download_done.emit(saved)
        except Exception as e:
            self._signals.error.emit(str(e))

    def _on_download_done(self, saved_path: str):
        """下載完成 / Download complete."""
        self._set_loading(False)
        self._download_btn.setEnabled(True)
        self._status.setText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_status_installed", "Installed: {path}").format(path=saved_path))

        msg = QMessageBox(self)
        msg.setWindowTitle(
            language_wrapper.language_word_dict.get("plugin_browser_download_btn", "Download & Install"))
        msg.setText(
            language_wrapper.language_word_dict.get(
                "plugin_browser_restart_hint",
                "Plugin downloaded to:\n{path}\n\nPlease restart the editor to activate."
            ).format(path=saved_path))
        msg.exec()

    def _on_error(self, error_msg: str):
        """顯示錯誤 / Show error."""
        jeditor_logger.error(f"PluginBrowserWidget error: {error_msg}")
        self._set_loading(False)
        self._status.setText(f"Error: {error_msg}")

    # ----- Helpers -----

    def _set_loading(self, loading: bool):
        """切換載入狀態 / Toggle loading state."""
        self._progress.setVisible(loading)
        self._fetch_btn.setEnabled(not loading)

    @staticmethod
    def _get_plugins_dir() -> Path:
        """
        取得 jeditor_plugins/ 目錄路徑，若不存在則建立。
        Get jeditor_plugins/ directory path, create if not exists.
        """
        plugins_dir = Path.cwd() / "jeditor_plugins"
        plugins_dir.mkdir(exist_ok=True)
        return plugins_dir



english_word_dict = {
    # Main
    "application_name": "JEditor",
    # Tab
    "tab_name_editor": "Editor",
    "tab_name_web_browser": "Web Browser",
    "tab_name_frontengine": "FrontEngine",
    # Browser
    "browser_download_detail": "Download Detail",
    "browser_find_text": "Find Text",
    "browser_find_text_input": "Input a text to find",
    "browser_back_button": "Back",
    "browser_forward_button": "Forward",
    "browser_reload_button": "Reload",
    "browser_search_button": "Search",
    # Dialog
    "create_file_dialog_pushbutton": "Create File",
    "input_file_name_dialog_pushbutton": "Please enter right file name",
    "search_next_dialog_pushbutton": "Search next",
    "search_back_dialog_pushbutton": "Search back",
    "search_box_dialog_title": "Search Text",
    "add_ai_model_title": "Please fill in AI model information",
    "add_ai_model_pushbutton": "Add AI Model",
    # Editor
    "editor_code_result": "Code result",
    "editor_format_check": "Format checker",
    # Check Code Style Menu
    "check_code_style_menu_label": "Check Code Style",
    "yapf_reformat_label": "yapf",
    "python_format_checker": "Python format check",
    "reformat_json_label": "Reformat JSON",
    "python_format_checker_only_support_python_message": "Only support python file",
    # Dock Menu
    "dock_menu_label": "Dock",
    "dock_browser_label": "New Dock Browser",
    "dock_stackoverflow_label": "New Dock Stackoverflow",
    "dock_editor_label": "New Dock Editor",
    "dock_frontengine_label": "New Dock FrontEngine",
    "dock_ipython_label": "New Ipython(Jupyter)",
    "dock_editor_title": "Editor",
    "dock_frontengine_title": "FrontEngine",
    "dock_browser_title": "Browser",
    "dock_ipython_title": "IPython",
    "dock_editor_menu": "Editor",
    "dock_git_menu": "Git",
    "dock_tools_menu": "Tools",
    "dock_ai_menu": "AI",
    # File Menu
    "file_menu_label": "File",
    "file_menu_new_file_label": "New File",
    "file_menu_open_file_label": "Open File",
    "file_menu_open_folder_label": "Open Folder",
    "file_menu_save_file_label": "Save File",
    "file_menu_encoding_label": "Encodings",
    "file_menu_font_label": "Font",
    "file_menu_font_size_label": "Font Size",
    # Help Menu
    "help_menu_label": "Help",
    "help_menu_open_github_label": "GitHub",
    "help_menu_open_doc_label": "Doc",
    "help_menu_open_about_label": "About",
    # Python Env Menu
    "python_env_menu_label": "Python Env",
    "python_env_menu_create_venv_label": "Create venv",
    "python_env_menu_pip_upgrade_label": "pip upgrade package",
    "python_env_menu_pip_label": "pip package",
    "python_env_menu_choose_interpreter_label": "Choose python interpreter",
    "python_env_menu_creating_venv_message": "Creating venv please waiting for shell exit code.",
    "python_env_menu_venv_exists": "venv already exists.",
    "python_env_menu_please_create_venv": "Please create venv first.",
    "python_env_menu_install_package_messagebox_title": "Install Package",
    "python_env_menu_install_package_messagebox_label": "What Package you want to install",
    "python_env_menu_install_or_update_package_messagebox_label": "What Package you want to install or update",
    # Run Menu
    "run_menu_label": "Run",
    "run_menu_run_program_label": "Run Program",
    "run_menu_run_debugger": "Run Debugger",
    "run_menu_run_on_shell_label": "Run On Shell",
    "run_menu_clear_result_label": "Clean Result",
    "run_menu_stop_all_program_label": "Stop All Program",
    "run_menu_stop_program_label": "Stop current program",
    "run_menu_run_help_label": "Run Help",
    "run_menu_shell_help_label": "Shell Help",
    "please_stop_current_running_shell": "Please stop current running shell first.",
    "please_stop_current_running_program": "Please stop current running program first.",
    "run_menu_run_help_tip": """
If you are unable to run a Python program, please make sure you are have python interpreter,
And if python conflict please choose another interpreter.
""",
    "run_menu_shell_run_tip": """
When executing a shell command, if you encounter a decoding error, 
please make sure that the current encoding is consistent with the default encoding of the system shell.
""",
    # Debugger and menu
    "editor_debugger_input_title_label": "Debugger",
    "show_debugger_input": "Show debugger input",
    # Shell and menu
    "editor_shell_input_title_label": "Shell",
    "show_shell_input": "Show shell input",
    # Program and menu
    "show_program_input": "Show program input",
    "process_input_send_command": "send command",
    # Style Menu
    "style_menu_label": "UI Style",
    # Tab Menu
    "tab_menu_label": "Tab",
    "tab_menu_tools_submenu_label": "Tools Tab",
    "tab_menu_git_submenu_label": "Git Tab",
    "tab_menu_add_editor_label": "Add Editor Tab",
    "tab_menu_add_frontengine_label": "Add FrontEngine Tab",
    "tab_menu_add_web_label": "Add Web Browser Tab",
    "tab_menu_add_stackoverflow_label": "Add Stackoverflow Tab",
    "tab_menu_editor_tab_name": "Editor",
    "tab_menu_frontengine_tab_name": "FrontEngine",
    "tab_menu_web_tab_name": "WEB Browser",
    "tab_menu_stackoverflow_tab_name": "Stackoverflow",
    "tab_menu_ipython_tab_name": "IPython(Jupyter)",
    "tab_menu_chat_ui_tab_name": "ChatUI",
    "tab_menu_git_client_tab_name": "Git Client",
    "tab_menu_git_branch_tree_view_tab_name": "Git BranchTreeViewer",
    "tab_menu_variable_inspector_tab_name": "Variable Inspector",
    "tab_menu_console_widget_tab_name": "Console Widget",
    "tab_code_diff_viewer_tab_name": "Code Diff Viewer",
    # Text Menu
    "text_menu_label": "Text",
    "text_menu_label_font": "Font",
    "text_menu_label_font_size": "Font Size",
    # System tray
    "system_tray_hide": "Hide",
    "system_tray_maximized": "Maximized",
    "system_tray_normal": "Normal",
    "system_tray_close": "Close",
    # Language Menu
    "language_menu_label": "Language",
    "language_menu_bar_english": "English",
    "language_menu_bar_traditional_chinese": "Traditional Chinese",
    "language_menu_bar_please_restart_messagebox": "Please restart",
    # Qtconsole
    "please_install_qtcontsole_label": "Please python -m pip install qtconsole first.",
    # Chat UI
    "font_size": "Font Size",
    "set_ai_model_waring_title": "Can not set AI Model",
    "set_ai_model_waring_text": "Please check [ai_base_url, ai_api_key, chat_model]",
    "call_ai_model_error_title": "Can't connect to AI Model",
    "call_ai_model_error_text": "Please check [ai_base_url, ai_api_key, chat_model]",
    "chat_ui_dock_label": "Chat UI",
    "chat_ui_set_ai_button": "Set AI setting",
    "chat_ui_load_ai_button": "Load AI setting",
    "chat_ui_call_ai_model_button": "Send prompt",
    "base_url_label": "AI server URL",
    "api_key_label": "AI server API Key",
    "ai_model_label": "AI Model",
    "load_ai_messagebox_title": "Load complete",
    "load_ai_messagebox_text": "Load complete",
    # gitGUI
    "label_repo_initial": "Repo: -",
    "btn_open_repo": "Open Repo",
    "btn_switch_branch": "Switch",
    "btn_pull": "Pull",
    "btn_push": "Push",
    "btn_clone_remote": "Clone Remote Repo",
    "label_remote": "Remote:",
    "label_branch": "Branch:",
    "placeholder_commit_message": "Commit message...",
    "btn_stage_all": "Stage All",
    "btn_commit": "Commit",
    "label_message": "Message:",
    "dialog_choose_repo": "Choose Git Repo",
    "err_open_repo": "Failed to open repository",
    "default_remote": "origin",
    "err_load_branches": "Failed to load branches",
    "err_load_commits": "Failed to load commits",
    "err_checkout": "Checkout Failure",
    "info_checkout_title": "Checkout",
    "info_checkout_msg": "Already check out to {branch}",
    "err_read_diff": "Failed to read diff",
    "err_stage": "Stage failed",
    "info_stage_title": "Stage",
    "info_stage_msg": "Executed git add -A",
    "err_commit": "Commit failed",
    "info_commit_title": "Commit",
    "info_commit_msg": "Commit created",
    "err_pull": "Pull failed",
    "info_pull_title": "Pull",
    "err_push": "Push failed",
    "info_push_title": "Push",
    "dialog_clone_title": "Clone Remote Repo",
    "dialog_clone_prompt": "Enter Git repository URL:",
    "dialog_select_folder": "Select Local Folder",
    "info_clone_success_title": "Clone Successful",
    "info_clone_success_msg": "Repository cloned to:\\n{repo_path}",
    "err_clone_failed_title": "Clone Failed",
    # Git graph
    "git_graph_title": "Git Branch Viewer",
    "git_graph_menu_file": "File",
    "git_graph_menu_open_repo": "Open Repository...",
    "git_graph_menu_refresh": "Refresh",
    "git_graph_menu_exit": "Exit",
    "git_graph_status_loading": "Loading repository...",
    "git_graph_status_ready": "Ready",
    "git_graph_status_repo_set": "Repository: {path}",
    "git_graph_error_not_git": "Selected path is not a Git repository.",
    "git_graph_error_exec_failed": "Git command failed.",
    "git_graph_tooltip_commit": "Commit: {short}\nAuthor: {author}\nDate: {date}\nMessage: {msg}",
    "git_graph_toolbar_open": "Open",
    "git_graph_toolbar_refresh": "Refresh",
    # Variable inspector
    "variable_inspector_title": "Variable Inspector",
    "variable_inspector_search": "Search Variables:",
    "variable_inspector_var_name": "Variable Name",
    "variable_inspector_var_type": "Type",
    "variable_inspector_var_value": "Value",
    # Dynamic Console
    "dynamic_console_title": "Dynamic Console",
    "dynamic_console_run": "Run",
    "dynamic_console_stop": "Stop",
    "dynamic_console_clear": "Clear",
    "dynamic_console_cwd": "CWD",
    "dynamic_console_shell": "Shell",
    "dynamic_console_prompt": "❯ ",
    "dynamic_console_running": "[running]",
    "dynamic_console_ready": "[ready]",
    "dynamic_console_process_running": "Process is running; stop it first",
    "dynamic_console_done": "[done] exit={code}, status={status}",
    "dynamic_console_system_prefix": "[system] ",
    # Code diff viewer
    # Plugin Menu
    "plugin_menu_label": "Plugins",
    "plugin_menu_about": "About",
    "plugin_menu_run_with": "Run with {name}",
    # Plugin Browser
    "plugin_browser_tab_name": "Plugin Browser",
    "plugin_browser_repo_label": "Repository URL:",
    "plugin_browser_fetch_btn": "Fetch Plugins",
    "plugin_browser_col_name": "Plugin",
    "plugin_browser_col_path": "Path",
    "plugin_browser_col_size": "Size",
    "plugin_browser_select_hint": "Select a plugin to view details",
    "plugin_browser_download_btn": "Download && Install",
    "plugin_browser_loading_source": "Loading source code...",
    "plugin_browser_overwrite_title": "Plugin exists",
    "plugin_browser_overwrite_msg": "{name} already exists. Overwrite?",
    "plugin_browser_invalid_url": "Invalid repository URL",
    "plugin_browser_status_ready": "Ready",
    "plugin_browser_status_fetching": "Fetching plugin list...",
    "plugin_browser_status_loaded": "Loaded {count} plugins",
    "plugin_browser_status_downloading": "Downloading {name}...",
    "plugin_browser_status_installed": "Installed: {path}",
    "plugin_browser_restart_hint": "Plugin downloaded to:\n{path}\n\nPlease restart the editor to activate.",
}



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from je_editor.pyside_ui.browser.main_browser_widget import MainBrowserWidget

from PySide6.QtCore import Qt
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QWidget, QPushButton, QGridLayout, QInputDialog

from je_editor.pyside_ui.browser.browser_serach_lineedit import BrowserLineSearch
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

from je_editor.pyside_ui.browser.browser_view import BrowserView


class BrowserWidget(QWidget):
    def __init__(self, start_url: str = "https://www.google.com/",
                 search_prefix: str = "https://www.google.com.tw/search?q=",
                 main_widget: MainBrowserWidget = None, browser_view: BrowserView = None):
        # --- Browser setting / 瀏覽器設定 ---
        super().__init__()
        self.main_widget = main_widget
        if browser_view:
            self.browser = browser_view
        else:
            # 建立內嵌的瀏覽器視圖
            # Create embedded browser view
            self.browser = BrowserView(start_url, main_widget=main_widget)
        self.search_prefix = search_prefix  # 搜尋引擎前綴字串 / Search engine prefix
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)  # 視窗關閉時釋放資源 / Free resources on close

        # --- Top bar buttons / 上方工具列按鈕 ---
        # 返回上一頁 / Back button
        self.back_button = QPushButton(language_wrapper.language_word_dict.get("browser_back_button"))
        self.back_button.clicked.connect(self.browser.back)

        # 前往下一頁 / Forward button
        self.forward_button = QPushButton(language_wrapper.language_word_dict.get("browser_forward_button"))
        self.forward_button.clicked.connect(self.browser.forward)

        # 重新整理 / Reload button
        self.reload_button = QPushButton(language_wrapper.language_word_dict.get("browser_reload_button"))
        self.reload_button.clicked.connect(self.browser.reload)

        # 搜尋按鈕 / Search button
        self.search_button = QPushButton(language_wrapper.language_word_dict.get("browser_search_button"))
        self.search_button.clicked.connect(self.search)

        # URL / Search input line (自訂義 QLineEdit)
        # Custom QLineEdit for URL or search input
        self.url_input = BrowserLineSearch(self)

        # --- Action: Ctrl+F to find text / 快捷鍵 Ctrl+F 搜尋文字 ---
        self.find_action = QAction(self)
        self.find_action.setShortcut("Ctrl+f")  # 設定快捷鍵 / Set shortcut
        self.find_action.triggered.connect(self.find_text)  # 綁定搜尋文字功能 / Connect to find_text
        self.addAction(self.find_action)

        # --- Layout / 版面配置 ---
        self.grid_layout = QGridLayout()
        self.grid_layout.addWidget(self.back_button, 0, 0)
        self.grid_layout.addWidget(self.forward_button, 0, 1)
        self.grid_layout.addWidget(self.reload_button, 0, 2)
        self.grid_layout.addWidget(self.url_input, 0, 3)
        self.grid_layout.addWidget(self.search_button, 0, 4)
        # 瀏覽器視圖佔滿下方所有空間
        # Browser view fills the lower area
        self.grid_layout.addWidget(self.browser, 1, 0, -1, -1)
        self.setLayout(self.grid_layout)

    def search(self):
        """
        Perform a search using the text in the input line.
        使用輸入框的文字進行搜尋，將字串附加到 search_prefix 後送出。
        """
        jeditor_logger.info("BrowserWidget Search")
        # 將輸入框文字拼接到搜尋前綴，並設定為瀏覽器 URL
        # Append input text to search prefix and set as browser URL
        self.browser.setUrl(f"{self.search_prefix}{self.url_input.text()}")

    def find_text(self):
        """
        Open a dialog to find text in the current page.
        開啟輸入對話框，在當前頁面中搜尋文字。
        - 如果按下 OK，搜尋輸入的文字
        - 如果取消，清除搜尋
        """
        jeditor_logger.info("BrowserWidget Find Text")
        search_box = QInputDialog(self)
        search_text, press_ok = search_box.getText(
            self,
            language_wrapper.language_word_dict.get("browser_find_text"),
            language_wrapper.language_word_dict.get("browser_find_text_input")
        )
        if press_ok:
            # 在頁面中搜尋輸入文字 / Search entered text in page
            self.browser.findText(search_text)
        else:
            # 清除搜尋結果 / Clear search
            self.browser.findText("")



from PySide6.QtCore import Qt
from PySide6.QtWebEngineCore import QWebEngineDownloadRequest
from PySide6.QtWidgets import QWidget, QBoxLayout, QPlainTextEdit

from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class BrowserDownloadWindow(QWidget):
    """
    A window widget to display details of a browser download.
    瀏覽器下載視窗，用來顯示下載的詳細資訊。
    """

    def __init__(self, download_instance: QWebEngineDownloadRequest):
        """
        Initialize the download window with a given QWebEngineDownloadRequest.
        使用指定的 QWebEngineDownloadRequest 初始化下載視窗。
        """
        super().__init__()
        # 記錄初始化訊息到 logger
        jeditor_logger.info("Init BrowserDownloadWindow "
                            f"download_instance: {download_instance}")

        # 設定視窗屬性：當視窗關閉時自動刪除
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)

        # 建立垂直方向的 BoxLayout
        self.box_layout = QBoxLayout(QBoxLayout.Direction.TopToBottom)

        # 建立文字框來顯示下載細節，並設為唯讀
        self.show_download_detail_plaintext = QPlainTextEdit()
        self.show_download_detail_plaintext.setReadOnly(True)

        # 設定視窗標題，支援多語言
        self.setWindowTitle(language_wrapper.language_word_dict.get("browser_download_detail"))

        # 儲存下載實例
        self.download_instance = download_instance

        # 綁定下載事件到對應的處理函式
        self.download_instance.isFinishedChanged.connect(self.print_finish)  # 當下載完成時
        self.download_instance.interruptReasonChanged.connect(self.print_interrupt)  # 當下載被中斷時
        self.download_instance.stateChanged.connect(self.print_state)  # 當下載狀態改變時

        # 接受下載請求，開始下載
        self.download_instance.accept()

        # 將文字框加入版面配置
        self.box_layout.addWidget(self.show_download_detail_plaintext)
        self.setLayout(self.box_layout)

    def print_finish(self):
        """
        Slot function triggered when download finishes.
        當下載完成時觸發，將完成狀態輸出到 logger 與文字框。
        """
        jeditor_logger.info("BrowserDownloadWindow Print Download is Finished")
        self.show_download_detail_plaintext.appendPlainText(str(self.download_instance.isFinished()))

    def print_interrupt(self):
        """
        Slot function triggered when download is interrupted.
        當下載被中斷時觸發，將中斷原因輸出到 logger 與文字框。
        """
        jeditor_logger.info("BrowserDownloadWindow Print interruptReason")
        self.show_download_detail_plaintext.appendPlainText(str(self.download_instance.interruptReason()))

    def print_state(self):
        """
        Slot function triggered when download state changes.
        當下載狀態改變時觸發，將狀態輸出到 logger 與文字框。
        """
        jeditor_logger.info("BrowserDownloadWindow Print State")
        self.show_download_detail_plaintext.appendPlainText(str(self.download_instance.state()))



import os

from PySide6.QtCore import Qt, QEvent
from PySide6.QtGui import QTextCursor, QColor, QKeyEvent
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QPlainTextEdit, QLineEdit,
    QPushButton, QLabel, QFileDialog, QComboBox
)

from je_editor.pyside_ui.main_ui.console_widget.qprocess_adapter import ConsoleProcessAdapter
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class ConsoleWidget(QWidget):
    """
    ConsoleWidget 提供一個互動式終端機介面，讓使用者可以輸入指令並查看輸出。
    ConsoleWidget provides an interactive console interface for running commands and viewing output.
    """

    def __init__(self, parent=None):
        super().__init__(parent)
        self.language_word_dict_get = language_wrapper.language_word_dict.get

        # 初始化子程序控制器 / Initialize process adapter
        self.proc = ConsoleProcessAdapter(self)
        self.proc.stdout.connect(self.append_text)  # 標準輸出 / Standard output
        self.proc.stderr.connect(lambda t: self.append_text(t, "#d33"))  # 錯誤輸出 (紅色) / Error output (red)
        self.proc.system.connect(
            lambda t: self.append_text(f"{self.language_word_dict_get('dynamic_console_system_prefix')}{t}\n", "#888")
        )
        self.proc.started.connect(lambda: self.proc.system.emit(self.language_word_dict_get("dynamic_console_running")))
        self.proc.finished.connect(self.on_finished)

        # 指令歷史紀錄 / Command history
        self.history, self.history_index = [], -1

        # 輸出區域 / Output area
        self.output = QPlainTextEdit(readOnly=True)
        self.output.setMaximumBlockCount(10000)  # 最多保留 10000 行 / Keep up to 10000 lines
        self.output.setStyleSheet("font-family: Consolas, monospace;")

        # 輸入框 / Input field
        self.input = QLineEdit()
        self.input.setPlaceholderText("Enter command")  # 預設提示文字 / Placeholder text
        self.input.returnPressed.connect(self.run_command)  # 按 Enter 執行指令 / Run command on Enter
        self.input.installEventFilter(self)  # 安裝事件過濾器，用於上下鍵瀏覽歷史 / Install event filter for history navigation

        # 控制按鈕 / Control buttons
        self.btn_run = QPushButton(self.language_word_dict_get("dynamic_console_run"))
        self.btn_run.clicked.connect(self.run_command)

        self.btn_stop = QPushButton(self.language_word_dict_get("dynamic_console_stop"))
        self.btn_stop.clicked.connect(self.proc.stop)

        self.btn_clear = QPushButton(self.language_word_dict_get("dynamic_console_clear"))
        self.btn_clear.clicked.connect(self.output.clear)

        # 工作目錄顯示與選擇 / Current working directory display and picker
        self.cwd_label = QLabel(f"{self.language_word_dict_get('dynamic_console_cwd')}: {os.getcwd()}")
        self.btn_pick_cwd = QPushButton("…")
        self.btn_pick_cwd.clicked.connect(self.pick_cwd)

        # Shell 選擇下拉選單 / Shell selection combobox
        self.shell_combo = QComboBox()
        self.shell_combo.addItems(["auto", "cmd", "powershell", "bash", "sh"])

        # 版面配置：上方 (工作目錄 + Shell 選擇) / Layout: Top (CWD + Shell selection)
        top = QHBoxLayout()
        top.addWidget(self.cwd_label)
        top.addWidget(self.btn_pick_cwd)
        top.addStretch()
        top.addWidget(QLabel(self.language_word_dict_get("dynamic_console_shell")))
        top.addWidget(self.shell_combo)

        # 版面配置：中間 (輸入框 + 按鈕) / Layout: Middle (Input + Buttons)
        mid = QHBoxLayout()
        mid.addWidget(self.input, 1)
        mid.addWidget(self.btn_run)
        mid.addWidget(self.btn_stop)
        mid.addWidget(self.btn_clear)

        # 主版面配置 / Main layout
        lay = QVBoxLayout(self)
        lay.addLayout(top)
        lay.addWidget(self.output, 1)
        lay.addLayout(mid)

        # 初始化狀態訊息 / Initial status message
        self.proc.system.emit(self.language_word_dict_get("dynamic_console_ready"))

        # 啟動互動式 shell / Start interactive shell
        self.proc.start_shell(self.shell_combo.currentText())

    # 事件過濾器：支援上下鍵瀏覽歷史指令
    # Event filter: Support navigating command history with Up/Down keys
    def eventFilter(self, obj, event):
        if obj is self.input and isinstance(event, QKeyEvent) and event.type() == QEvent.Type.KeyPress:
            if event.key() == Qt.Key.Key_Up:
                self.history_prev()
                return True
            if event.key() == Qt.Key.Key_Down:
                self.history_next()
                return True
        return super().eventFilter(obj, event)

    # 瀏覽上一個歷史指令 / Navigate to previous command in history
    def history_prev(self):
        if not self.history: return
        self.history_index = len(self.history) - 1 if self.history_index < 0 else max(0, self.history_index - 1)
        self.input.setText(self.history[self.history_index])
        self.input.end(False)

    # 瀏覽下一個歷史指令 / Navigate to next command in history
    def history_next(self):
        if not self.history: return
        if self.history_index < 0: return
        self.history_index += 1
        if self.history_index >= len(self.history):
            self.history_index = -1
            self.input.clear()
        else:
            self.input.setText(self.history[self.history_index])
            self.input.end(False)

    # 選擇新的工作目錄 / Pick a new working directory
    def pick_cwd(self):
        d = QFileDialog.getExistingDirectory(self, "Select working directory", os.getcwd())
        if d:
            self.proc.set_cwd(d)
            self.cwd_label.setText(f"{self.language_word_dict_get('dynamic_console_cwd')}: {d}")
            self.proc.system.emit(f'cd "{d}"')

    # 在輸出區域新增文字 (支援顏色) / Append text to output area (with optional color)
    def append_text(self, text, color=None):
        cursor = self.output.textCursor()
        cursor.movePosition(QTextCursor.MoveOperation.End)
        fmt = self.output.currentCharFormat()
        if color:
            fmt.setForeground(QColor(color))
        cursor.setCharFormat(fmt)
        cursor.insertText(text)
        self.output.setTextCursor(cursor)
        self.output.ensureCursorVisible()

    # 執行輸入的指令 / Run the entered command
    def run_command(self):
        cmd = self.input.text().strip()
        if not cmd: return
        if not self.history or self.history[-1] != cmd:
            self.history.append(cmd)
        self.history_index = -1
        self.append_text(f"{self.language_word_dict_get('dynamic_console_prompt')}{cmd}\n", "#0aa")
        self.proc.send_command(cmd)
        self.input.clear()

    # 子程序結束時的處理 / Handle process finished event
    def on_finished(self, code, status):
        self.append_text(
            f"\n{self.language_word_dict_get('dynamic_console_done').format(code=code, status=status)}\n",
            "#888"
        )
        self.proc.system.emit(self.language_word_dict_get("dynamic_console_ready"))



from je_editor.utils.logging.loggin_instance import jeditor_logger


# -----------------------------
# Programming Language Plugin Registry
# 程式語言插件註冊表
# -----------------------------
# Maps file suffix (e.g. ".cpp") to syntax config dict
# 將副檔名（如 ".cpp"）映射到語法設定字典
_programming_language_plugins: dict = {}


def register_programming_language(
    suffix: str,
    syntax_words: dict,
    syntax_rules: dict | None = None,
) -> None:
    """
    註冊一個程式語言插件，用於語法高亮。
    Register a programming language plugin for syntax highlighting.

    :param suffix: 副檔名，例如 ".cpp" / File suffix, e.g. ".cpp"
    :param syntax_words: 關鍵字設定字典 / Keyword settings dict
        格式 / Format: {"group_name": {"words": (...), "color": QColor(...)}}
    :param syntax_rules: 額外語法規則字典（可選）/ Extra syntax rules dict (optional)
        格式 / Format: {"rule_name": {"rules": (...), "color": QColor(...)}}
    """
    jeditor_logger.info(f"register_programming_language suffix: {suffix}")
    _programming_language_plugins[suffix] = {
        "syntax_words": syntax_words,
        "syntax_rules": syntax_rules or {},
    }
    # 同步到 syntax_extend_setting_dict，讓 PythonHighlighter 也能使用
    # Sync to syntax_extend_setting_dict so PythonHighlighter can use it
    from je_editor.pyside_ui.code.syntax.syntax_setting import syntax_extend_setting_dict
    syntax_extend_setting_dict[suffix] = syntax_words


def get_programming_language_plugin(suffix: str) -> dict | None:
    """
    取得指定副檔名的程式語言插件。
    Get programming language plugin by file suffix.
    """
    return _programming_language_plugins.get(suffix)


def get_all_programming_language_suffixes() -> list[str]:
    """
    取得所有已註冊的程式語言副檔名列表。
    Get all registered programming language suffixes.
    """
    return list(_programming_language_plugins.keys())


# -----------------------------
# Natural Language Plugin Registry
# 自然語言插件註冊表
# -----------------------------
# Maps language key (e.g. "French") to translation dict
# 將語言鍵（如 "French"）映射到翻譯字典
_natural_language_plugins: dict = {}


def register_natural_language(
    language_key: str,
    display_name: str,
    word_dict: dict,
) -> None:
    """
    註冊一個自然語言插件，用於 UI 翻譯。
    Register a natural language plugin for UI translation.

    :param language_key: 語言鍵，例如 "French" / Language key, e.g. "French"
    :param display_name: 顯示名稱，例如 "Francais" / Display name, e.g. "Francais"
    :param word_dict: 翻譯字典 / Translation dictionary (same keys as english_word_dict)
    """
    jeditor_logger.info(f"register_natural_language language_key: {language_key}")
    _natural_language_plugins[language_key] = {
        "display_name": display_name,
        "word_dict": word_dict,
    }
    # 同步到 LanguageWrapper，讓語言切換生效
    # Sync to LanguageWrapper so language switching works
    from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper
    language_wrapper.choose_language_dict[language_key] = word_dict


def get_natural_language_plugin(language_key: str) -> dict | None:
    """
    取得指定語言鍵的自然語言插件。
    Get natural language plugin by language key.
    """
    return _natural_language_plugins.get(language_key)


def get_all_natural_languages() -> dict:
    """
    取得所有已註冊的自然語言插件。
    Get all registered natural language plugins.
    Returns: {"French": {"display_name": "Francais", "word_dict": {...}}, ...}
    """
    return _natural_language_plugins.copy()


# -----------------------------
# Plugin Run Config Registry
# 插件執行設定註冊表
# -----------------------------
# Maps plugin name to run config dict
# 將插件名稱映射到執行設定字典
_plugin_run_configs: list[dict] = []


def register_plugin_run_config(run_config: dict) -> None:
    """
    註冊一個插件的執行設定。
    Register a plugin's run configuration.

    :param run_config: 執行設定字典 / Run configuration dict
        格式 / Format: {"name": "C++ (G++)", "suffixes": (".cpp",), "compiler": "g++", ...}
    """
    jeditor_logger.info(f"register_plugin_run_config: {run_config.get('name')}")
    _plugin_run_configs.append(run_config)


def get_all_plugin_run_configs() -> list[dict]:
    """
    取得所有已註冊的插件執行設定。
    Get all registered plugin run configurations.
    """
    return _plugin_run_configs.copy()


def get_plugin_run_config_by_suffix(suffix: str) -> dict | None:
    """
    依副檔名取得對應的插件執行設定。
    Get plugin run configuration by file suffix.
    """
    for config in _plugin_run_configs:
        if suffix in config.get("suffixes", ()):
            return config
    return None


# -----------------------------
# Plugin Metadata Registry
# 插件元資料註冊表
# -----------------------------
# Stores plugin metadata (name, author, version, etc.)
# 儲存插件元資料（名稱、作者、版本等）
_plugin_metadata_list: list[dict] = []


def register_plugin_metadata(metadata: dict) -> None:
    """
    註冊一個插件的元資料。
    Register a plugin's metadata.

    :param metadata: 元資料字典 / Metadata dict
        格式 / Format: {"name": "...", "author": "...", "version": "...", "run_config": {...} or None}
    """
    jeditor_logger.info(f"register_plugin_metadata: {metadata.get('name')}")
    _plugin_metadata_list.append(metadata)


def get_all_plugin_metadata() -> list[dict]:
    """
    取得所有已註冊的插件元資料。
    Get all registered plugin metadata.
    """
    return _plugin_metadata_list.copy()



"""
外部插件載入器 / External Plugin Loader

從工作目錄下的 jeditor_plugins/ 目錄自動載入插件。
Auto-discover and load plugins from jeditor_plugins/ under the working directory.

插件目錄結構 / Plugin directory structure:
    jeditor_plugins/
        my_language.py              # 單檔插件 / Single-file plugin
        my_package/                 # 套件插件 / Package plugin
            __init__.py
        program_languages/          # 分類目錄 / Category directory
            cpp_syntax.py
        languages/
            french.py

每個插件必須提供一個 register() 函式。
Each plugin must provide a register() function.
"""
import importlib.util
import sys
import traceback
from pathlib import Path

from je_editor.utils.logging.loggin_instance import jeditor_logger

_PACKAGE_ROOT = Path(__file__).resolve().parent.parent.parent
_PLUGIN_DIR_NAME = "jeditor_plugins"


def _find_all_plugins_dirs() -> list[Path]:
    """
    收集所有可能的 jeditor_plugins/ 目錄（去重）。
    Collect all possible jeditor_plugins/ directories (deduplicated).

    搜尋順序 / Search order:
    1. 使用者的工作目錄 / User's current working directory
    2. je_editor 套件的上層目錄（開發模式）/ Parent of je_editor package (development mode)

    兩個位置都會掃描，確保不同位置啟動都能載入插件。
    Both locations are scanned so plugins load regardless of launch location.
    """
    dirs = []
    seen = set()

    for base in (Path.cwd(), _PACKAGE_ROOT):
        candidate = (base / _PLUGIN_DIR_NAME).resolve()
        if candidate not in seen and candidate.exists():
            dirs.append(candidate)
            seen.add(candidate)

    return dirs


_plugins_loaded = False


def _collect_plugin_entries(directory: Path, entries: list[tuple[str, Path]]) -> None:
    """
    遞迴收集插件入口。
    Recursively collect plugin entries from a directory.

    支援三種結構 / Supports three structures:
    1. 單檔插件: xxx.py (含 register()) / Single-file plugin
    2. 套件插件: xxx/__init__.py / Package plugin
    3. 分類目錄: 不含 __init__.py 的子目錄，遞迴進入 / Category directory (no __init__.py), recurse into
    """
    if not directory.exists():
        return

    for item in sorted(directory.iterdir()):
        if item.name.startswith("_") or item.name.startswith("."):
            continue

        if item.is_file() and item.suffix == ".py":
            # 單檔插件 / Single-file plugin
            entries.append((item.stem, item))
        elif item.is_dir():
            if (item / "__init__.py").exists():
                # 套件插件 / Package plugin
                entries.append((item.name, item / "__init__.py"))
            else:
                # 分類目錄，遞迴進入 / Category directory, recurse
                _collect_plugin_entries(item, entries)


def load_external_plugins(plugins_dir: Path | str | None = None) -> list[str]:
    """
    從 jeditor_plugins/ 目錄載入所有外部插件。
    Load all external plugins from jeditor_plugins/ directory.

    每個插件（.py 檔案或含 __init__.py 的資料夾）都需提供 register() 函式。
    Each plugin (.py file or folder with __init__.py) must provide a register() function.

    :param plugins_dir: 插件目錄路徑（預設為專案根目錄下的 jeditor_plugins/）
                        Plugin directory path (defaults to jeditor_plugins/ under project root)
    :return: 成功載入的插件名稱列表 / List of successfully loaded plugin names
    """
    global _plugins_loaded
    if _plugins_loaded and plugins_dir is None:
        return []

    loaded = []

    # 決定要掃描的目錄 / Determine directories to scan
    if plugins_dir is not None:
        dirs_to_scan = [Path(plugins_dir)]
    else:
        dirs_to_scan = _find_all_plugins_dirs()

    if not dirs_to_scan:
        _plugins_loaded = True
        jeditor_logger.info("No plugin directories found, skipping external plugins")
        return loaded

    # 從所有目錄收集插件入口（遞迴掃描子目錄）
    # Collect plugin entries from all directories (recursively scan subdirectories)
    plugin_entries = []
    seen_names: set[str] = set()
    for scan_dir in dirs_to_scan:
        jeditor_logger.info(f"Loading external plugins from: {scan_dir}")
        entries: list[tuple[str, Path]] = []
        _collect_plugin_entries(scan_dir, entries)
        for name, path in entries:
            # 同名插件只載入第一個（工作目錄優先）
            # Skip duplicate plugin names (working directory takes priority)
            if name not in seen_names:
                plugin_entries.append((name, path))
                seen_names.add(name)

    # 載入並註冊 / Load and register
    for plugin_name, plugin_path in plugin_entries:
        try:
            jeditor_logger.info(f"Loading plugin: {plugin_name} from {plugin_path}")

            spec = importlib.util.spec_from_file_location(
                f"jeditor_plugin_{plugin_name}", str(plugin_path)
            )
            if spec is None or spec.loader is None:
                jeditor_logger.warning(
                    f"Plugin '{plugin_name}' cannot be loaded from {plugin_path}, skipped"
                )
                continue
            module = importlib.util.module_from_spec(spec)
            sys.modules[spec.name] = module
            spec.loader.exec_module(module)

            # 收集插件元資料 / Collect plugin metadata
            from je_editor.plugins import register_plugin_metadata, register_plugin_run_config
            metadata = {
                "name": getattr(module, "PLUGIN_NAME", plugin_name),
                "author": getattr(module, "PLUGIN_AUTHOR", ""),
                "version": getattr(module, "PLUGIN_VERSION", ""),
                "run_config": getattr(module, "PLUGIN_RUN_CONFIG", None),
            }
            register_plugin_metadata(metadata)

            # 註冊執行設定 / Register run config
            if hasattr(module, "PLUGIN_RUN_CONFIG"):
                register_plugin_run_config(module.PLUGIN_RUN_CONFIG)

            # 呼叫 register() 函式 / Call register() function
            if hasattr(module, "register"):
                module.register()
                loaded.append(plugin_name)
                jeditor_logger.info(f"Plugin loaded successfully: {plugin_name}")
            else:
                jeditor_logger.warning(
                    f"Plugin '{plugin_name}' has no register() function, skipped"
                )

        except Exception as e:
            jeditor_logger.error(f"Failed to load plugin '{plugin_name}': {e}")
            jeditor_logger.error(traceback.format_exc())

    _plugins_loaded = True
    jeditor_logger.info(f"External plugins loaded: {loaded}")
    return loaded



from PySide6.QtGui import QColor, QTextCursor, QTextCharFormat, QFont
from PySide6.QtWidgets import QPlainTextEdit, QTextEdit, QWidget, QHBoxLayout, QVBoxLayout, QLabel

from je_editor.pyside_ui.git_ui.code_diff_compare.line_number_code_viewer import LineNumberedCodeViewer


class SideBySideDiffWidget(QWidget):
    """
    Side-by-side diff viewer widget.
    左右對照的差異檢視元件。
    """

    def __init__(self, parent=None):
        super().__init__(parent)

        # === 顏色設定 / Color configuration ===
        self.is_dark = True
        # 刪除行背景 / Deleted line background
        self.color_del = QColor("#ffcccc")  # 淡紅色，深淺背景都清楚
        # 新增行背景 / Added line background
        self.color_add = QColor("#ccffcc")  # 淡綠色，對比度佳
        # Hunk header 背景
        self.color_hunk = QColor("#cce5ff")  # 淡藍色，醒目但不刺眼
        # Diff header 背景
        self.color_header = QColor("#e0e0e0")  # 淺灰，適合標題區塊

        # === 左右檔名標籤 / File name labels ===
        self.leftLabel = QLabel("Left: (old)")
        self.rightLabel = QLabel("Right: (new)")
        font = QFont()
        font.setBold(True)
        self.leftLabel.setFont(font)
        self.rightLabel.setFont(font)

        # === 左右文字編輯器 / Left and right code editors ===
        self.leftEdit = LineNumberedCodeViewer()
        self.rightEdit = LineNumberedCodeViewer()
        for edit in (self.leftEdit, self.rightEdit):
            edit.setReadOnly(True)  # 設為唯讀 / Read-only
            edit.setLineWrapMode(QPlainTextEdit.LineWrapMode.NoWrap)  # 不自動換行 / Disable line wrap
            mono = QFont("Consolas")  # 使用等寬字型 / Monospaced font
            mono.setStyleHint(QFont.StyleHint.Monospace)
            edit.setFont(mono)

        # === 版面配置 / Layout ===
        leftBox = QVBoxLayout()
        leftBox.addWidget(self.leftLabel)
        leftBox.addWidget(self.leftEdit)

        rightBox = QVBoxLayout()
        rightBox.addWidget(self.rightLabel)
        rightBox.addWidget(self.rightEdit)

        main = QHBoxLayout(self)
        leftContainer = QWidget(self)
        leftContainer.setLayout(leftBox)
        rightContainer = QWidget(self)
        rightContainer.setLayout(rightBox)
        main.addWidget(leftContainer)
        main.addWidget(rightContainer)

        # 同步左右捲軸 / Sync scrollbars
        self._sync_scrollbars()

        # 預設深色模式 / Default to dark theme
        self.set_dark_theme()

    def _sync_scrollbars(self):
        """
        Synchronize scrollbars between left and right editors.
        同步左右編輯器的捲軸。
        """
        self.leftEdit.verticalScrollBar().valueChanged.connect(
            self.rightEdit.verticalScrollBar().setValue
        )
        self.rightEdit.verticalScrollBar().valueChanged.connect(
            self.leftEdit.verticalScrollBar().setValue
        )
        self.leftEdit.horizontalScrollBar().valueChanged.connect(
            self.rightEdit.horizontalScrollBar().setValue
        )
        self.rightEdit.horizontalScrollBar().valueChanged.connect(
            self.leftEdit.horizontalScrollBar().setValue
        )

    def set_diff_text(self, diff_text: str):
        """
        Parse unified diff text and display it in side-by-side editors.
        解析 unified diff 文字並顯示在左右編輯器。
        """
        left_lines, right_lines, left_marks, right_marks, left_name, right_name = \
            self._parse_unified_diff(diff_text)

        self.leftLabel.setText(f"Left: {left_name or '(old)'}")
        self.rightLabel.setText(f"Right: {right_name or '(new)'}")

        self._set_text_with_highlights(self.leftEdit, left_lines, left_marks)
        self._set_text_with_highlights(self.rightEdit, right_lines, right_marks)

        # 游標移到開頭 / Move cursor to start
        self.leftEdit.moveCursor(QTextCursor.MoveOperation.Start)
        self.rightEdit.moveCursor(QTextCursor.MoveOperation.Start)

    def _set_text_with_highlights(self, edit: QPlainTextEdit, lines, marks):
        """
        Set text and apply syntax highlighting based on diff marks.
        設定文字並依 diff 標記加上背景色。
        """
        edit.setPlainText("\n".join(lines))

        diff_extras = []
        for i, mark in enumerate(marks):
            fmt = QTextCharFormat()
            # Always set foreground so it won't fall back
            # 永遠設定前景色，避免 fallback
            fmt.setForeground(QColor("#d4d4d4") if self.is_dark else QColor("black"))

            if mark == "DEL":
                fmt.setBackground(self.color_del)
            elif mark == "ADD":
                fmt.setBackground(self.color_add)
            elif mark == "HUNK":
                fmt.setBackground(self.color_hunk)
            elif mark == "HDR":
                fmt.setBackground(self.color_header)
            else:
                continue

            sel = self._line_selection(edit, i, fmt)
            diff_extras.append(sel)

        # 保留 diff selections，方便主題切換時重用
        setattr(edit, "_diff_extras", diff_extras)

        # 嘗試合併其他高亮（例如 LineNumberedCodeViewer 的當前行高亮）
        if hasattr(edit, "_current_line_extras"):
            merged = diff_extras + edit._current_line_extras
        else:
            merged = diff_extras

        edit.setExtraSelections(merged)

    def _line_selection(self, edit: QPlainTextEdit, line_index: int, fmt: QTextCharFormat):
        """
        Create a selection for a specific line with given format.
        建立某一行的選取區並套用格式。
        """
        sel = QTextEdit.ExtraSelection()
        sel.format = fmt
        cursor = edit.textCursor()
        cursor.movePosition(QTextCursor.MoveOperation.Start)
        for _ in range(line_index):
            cursor.movePosition(QTextCursor.MoveOperation.Down)
        cursor.select(QTextCursor.SelectionType.LineUnderCursor)
        sel.cursor = cursor
        return sel

    def _parse_unified_diff(self, diff_text: str):
        """
        Parse unified diff into left/right lines and marks.
        將 unified diff 解析成左右行與標記。
        """
        left_lines, right_lines, left_marks, right_marks = [], [], [], []
        left_name, right_name = None, None

        def add_left(line, mark=None):
            left_lines.append(line)
            left_marks.append(mark or "CTX")

        def add_right(line, mark=None):
            right_lines.append(line)
            right_marks.append(mark or "CTX")

        def align():
            # 對齊左右行數 / Align left and right line counts
            if len(left_lines) < len(right_lines):
                for _ in range(len(right_lines) - len(left_lines)):
                    add_left("")
            elif len(right_lines) < len(left_lines):
                for _ in range(len(left_lines) - len(right_lines)):
                    add_right("")

        for raw in diff_text.splitlines():
            if raw.startswith("diff "):
                add_left(raw, "HDR");
                add_right(raw, "HDR");
                align()
            elif raw.startswith("--- "):
                left_name = raw[4:].strip()
                add_left(raw, "HDR");
                add_right("", "HDR");
                align()
            elif raw.startswith("+++ "):
                right_name = raw[4:].strip()
                add_left("", "HDR");
                add_right(raw, "HDR");
                align()
            elif raw.startswith("@@"):
                add_left(raw, "HUNK");
                add_right(raw, "HUNK");
                align()
            elif raw.startswith("-"):
                add_left(raw, "DEL");
                add_right("", None);
                align()
            elif raw.startswith("+"):
                add_left("", None);
                add_right(raw, "ADD");
                align()
            else:
                add_left(raw, None);
                add_right(raw, None);
                align()

        return left_lines, right_lines, left_marks, right_marks, left_name, right_name

    def _reapply_highlights_for_theme(self):
        """
        Reapply highlights when theme changes.
        主題切換時重新套用高亮。
        """
        for edit in (self.leftEdit, self.rightEdit):
            if hasattr(edit, "_diff_extras"):
                updated = []
                for sel in edit._diff_extras:
                    fmt: QTextCharFormat = QTextCharFormat(sel.format)

                    # 前景色依主題切換
                    fmt.setForeground(QColor("#d4d4d4") if self.is_dark else QColor("black"))

                    # 依照 mark 更新背景色
                    cursor = sel.cursor
                    cursor.select(QTextCursor.SelectionType.LineUnderCursor)
                    text = cursor.selectedText()

                    if text.startswith("-"):
                        fmt.setBackground(self.color_del)
                    elif text.startswith("+"):
                        fmt.setBackground(self.color_add)
                    elif text.startswith("@@"):
                        fmt.setBackground(self.color_hunk)
                    elif text.startswith("diff") or text.startswith("---") or text.startswith("+++"):
                        fmt.setBackground(self.color_header)

                    sel.format = fmt
                    updated.append(sel)

                edit._diff_extras = updated

                if hasattr(edit, "_current_line_extras"):
                    merged = updated + edit._current_line_extras
                else:
                    merged = updated
                edit.setExtraSelections(merged)

    def set_dark_theme(self):
        """
        Apply dark theme colors.
        套用深色主題配色。
        """
        self.is_dark = True
        self.color_del = QColor(60, 20, 20)
        self.color_add = QColor(20, 60, 20)
        self.color_hunk = QColor(25, 25, 60)
        self.color_header = QColor(50, 50, 50)
        self.setStyleSheet("""QWidget { background-color: #1e1e1e; color: #d4d4d4; }""")
        self._reapply_highlights_for_theme()
        self.leftEdit.apply_theme_to_editor(dark=self.is_dark)
        self.rightEdit.apply_theme_to_editor(dark=self.is_dark)

    def set_light_theme(self):
        """
        Apply light  theme colors.
        套用淺色主題配色。
        """
        self.is_dark = False
        self.color_del = QColor(255, 230, 230)
        self.color_add = QColor(230, 255, 230)
        self.color_hunk = QColor(230, 230, 255)
        self.color_header = QColor(240, 240, 240)
        self.setStyleSheet("""QWidget { background-color: white; color: black; }""")
        self._reapply_highlights_for_theme()
        self.leftEdit.apply_theme_to_editor(dark=self.is_dark)
        self.rightEdit.apply_theme_to_editor(dark=self.is_dark)



from os import getcwd
from pathlib import Path
from typing import Dict

from PySide6.QtGui import QColor

from je_editor.pyside_ui.main_ui.save_settings.setting_utils import write_setting
from je_editor.utils.json.json_file import read_json
from je_editor.utils.logging.loggin_instance import jeditor_logger


def update_actually_color_dict():
    """
    更新實際使用的顏色字典 (actually_color_dict)，
    將 user_setting_color_dict 中的 RGB 值轉換成 QColor 物件。
    Update the actual color dictionary (actually_color_dict),
    converting RGB values from user_setting_color_dict into QColor objects.
    """
    jeditor_logger.info(f"user_color_setting_file.py update_actually_color_dict")

    def _to_qcolor(key: str, fallback: list) -> QColor:
        """Safely convert RGB list to QColor with fallback."""
        rgb = user_setting_color_dict.get(key, fallback)
        if rgb is None or len(rgb) < 3:
            rgb = fallback
        return QColor(rgb[0], rgb[1], rgb[2])

    actually_color_dict.update(
        {
            "line_number_color": _to_qcolor("line_number_color", [255, 255, 255]),
            "line_number_background_color": _to_qcolor("line_number_background_color", [179, 179, 179]),
            "current_line_color": _to_qcolor("current_line_color", [148, 148, 184]),
            "normal_output_color": _to_qcolor("normal_output_color", [255, 255, 255]),
            "error_output_color": _to_qcolor("error_output_color", [255, 0, 0]),
            "warning_output_color": _to_qcolor("warning_output_color", [204, 204, 0]),
        }
    )


# 使用者設定的顏色字典 (以 RGB list 表示)
# User-defined color dictionary (stored as RGB lists)
user_setting_color_dict: Dict[str, list] = {
    "line_number_color": [255, 255, 255],
    "line_number_background_color": [179, 179, 179],
    "current_line_color": [148, 148, 184],
    "normal_output_color": [255, 255, 255],
    "error_output_color": [255, 0, 0],
    "warning_output_color": [204, 204, 0]
}

# 實際使用的顏色字典 (以 QColor 表示)
# Actual color dictionary (stored as QColor objects)
actually_color_dict: Dict[str, QColor] = {}

# 初始化時先更新一次
# Update once at initialization
update_actually_color_dict()


def write_user_color_setting() -> None:
    """
    將使用者顏色設定寫入 JSON 檔案
    Write user color settings into a JSON file
    """
    jeditor_logger.info("user_color_setting_file.py write_user_color_setting")
    write_setting(user_setting_color_dict, "user_color_setting.json")


def read_user_color_setting() -> None:
    """
    從 JSON 檔案讀取使用者顏色設定，並更新 user_setting_color_dict
    Read user color settings from JSON file and update user_setting_color_dict
    """
    jeditor_logger.info("user_color_setting_file.py read_user_color_setting")
    user_color_setting_file = Path(getcwd()) / ".jeditor" / "user_color_setting.json"

    # 如果檔案存在，就讀取並更新設定
    # If the file exists, read and update the settings
    if user_color_setting_file.exists() and user_color_setting_file.is_file():
        data = read_json(str(user_color_setting_file))
        if data is not None:
            user_setting_color_dict.update(data)



# 匯入未來功能，允許延遲型別註解 (Python 3.7+ 常用)
# Import future feature: postponed evaluation of type annotations
from __future__ import annotations

# 僅用於型別檢查，避免循環匯入
# For type checking only (avoids circular imports)
from typing import TYPE_CHECKING

# 匯入 Qt 動作
# Import QAction
from PySide6.QtGui import QAction

# 匯入程式執行管理器 (ExecManager)
# Import ExecManager for running code
from je_editor.pyside_ui.code.code_process.code_exec import ExecManager
# 匯入檔案儲存對話框
# Import file save dialog
from je_editor.pyside_ui.dialog.file_dialog.save_file_dialog import choose_file_get_save_file_path
# 匯入編輯器元件
# Import EditorWidget
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
# 匯入輸入處理 (用於程式輸入)
# Import ProcessInput for handling program input
from je_editor.pyside_ui.main_ui.editor.process_input import ProcessInput
# 匯入工具函式：顯示「請先關閉目前執行中的程式」訊息框
# Import utility: show message box if a program is already running
from je_editor.pyside_ui.main_ui.menu.run_menu.under_run_menu.utils import please_close_current_running_messagebox
# 匯入日誌紀錄器
# Import logger instance
from je_editor.utils.logging.loggin_instance import jeditor_logger
# 匯入多語言包裝器
# Import multi-language wrapper for UI localization
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

# 僅在型別檢查時匯入 EditorMain，避免循環依賴
# Import EditorMain only for type checking (avoids circular dependency)
if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


# 設定「執行程式」選單
# Set up the "Run Program" menu
def set_program_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_program_menu.py set_program_menu ui_we_want_to_set: {ui_we_want_to_set}")
    # 在 Run 選單下建立 Program 子選單
    # Create Program submenu under Run menu
    ui_we_want_to_set.run_program_menu = ui_we_want_to_set.run_menu.addMenu(
        language_wrapper.language_word_dict.get("run_menu_run_program_label"))

    # 建立「執行程式」動作
    # Add "Run Program" action
    ui_we_want_to_set.run_program_menu.run_program_action = QAction(
        language_wrapper.language_word_dict.get("run_menu_run_program_label"))
    ui_we_want_to_set.run_program_menu.run_program_action.triggered.connect(
        lambda: run_program(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_program_menu.addAction(ui_we_want_to_set.run_program_menu.run_program_action)

    # 建立「顯示程式輸入」動作
    # Add "Show Program Input" action
    ui_we_want_to_set.run_program_menu.show_shell_input = QAction(
        language_wrapper.language_word_dict.get("show_program_input"))
    ui_we_want_to_set.run_program_menu.show_shell_input.triggered.connect(
        lambda: show_program_input(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_program_menu.addAction(ui_we_want_to_set.run_program_menu.show_shell_input)


# 執行程式
# Run program
def run_program(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_program_menu.py run_program ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        # 確保沒有正在執行的程式
        # Ensure no program is already running
        if widget.exec_program is None:
            widget.python_compiler = ui_we_want_to_set.python_compiler
            # 要求使用者選擇儲存檔案路徑
            # Ask user to choose save file path
            if choose_file_get_save_file_path(ui_we_want_to_set):
                # 檢查是否有對應的插件執行設定
                # Check if there's a matching plugin run config for the file suffix
                from pathlib import Path
                from je_editor.plugins import get_plugin_run_config_by_suffix
                file_suffix = Path(widget.current_file).suffix if widget.current_file else ""
                plugin_config = get_plugin_run_config_by_suffix(file_suffix)

                # 建立程式執行管理器並執行程式
                # Create ExecManager and run program
                code_exec = ExecManager(widget, program_encoding=ui_we_want_to_set.encoding)
                code_exec.later_init()

                if plugin_config is not None:
                    # 使用插件執行設定 / Use plugin run config
                    code_exec.exec_with_plugin_config(widget.current_file, plugin_config)
                else:
                    # 預設 Python 執行 / Default Python execution
                    code_exec.exec_code(widget.current_file)

                widget.exec_program = code_exec
        else:
            # 如果已有程式在執行，顯示提示訊息
            # If a program is already running, show message
            please_close_current_running_messagebox(ui_we_want_to_set)


# 顯示程式輸入介面
# Show program input interface
def show_program_input(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_program_menu.py show_program_input ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.program_input = ProcessInput(widget, "program")
        widget.program_input.show()



from os import getcwd
from pathlib import Path

from je_editor.pyside_ui.main_ui.save_settings.setting_utils import write_setting
from je_editor.utils.json.json_file import read_json
from je_editor.utils.logging.loggin_instance import jeditor_logger

# 匯入通用的設定寫入工具
# Import utility function to write settings
# 匯入 JSON 讀取工具
# Import JSON reader utility

# 匯入日誌工具
# Import logger


# 使用者設定字典，包含 UI 與編輯器的偏好
# User settings dictionary, storing UI and editor preferences
user_setting_dict = {
    "ui_font": "Lato",  # UI 字型 / UI font
    "ui_font_size": 12,  # UI 字體大小 / UI font size
    "language": "English",  # 語言設定 / Language setting
    "ui_style": "dark_amber.xml",  # UI 樣式檔 / UI style file
    "font": "Lato",  # 編輯器字型 / Editor font
    "font_size": 12,  # 編輯器字體大小 / Editor font size
    "encoding": "utf-8",  # 檔案編碼 / File encoding
    "last_file": None,  # 上次開啟的檔案 / Last opened file
    "python_compiler": None,  # Python 編譯器路徑 / Python compiler path
    "max_line_of_output": 200000,  # 最大輸出行數限制 / Max lines of output
}


def write_user_setting() -> None:
    """
    將使用者設定寫入 JSON 檔案
    Write user settings into a JSON file
    """
    jeditor_logger.info("user_setting_file.py write_user_setting")
    # 呼叫通用工具，將 user_setting_dict 寫入 `.jeditor/user_setting.json`
    # Use utility to write user_setting_dict into `.jeditor/user_setting.json`
    write_setting(user_setting_dict, "user_setting.json")


def read_user_setting() -> None:
    """
    從 JSON 檔案讀取使用者設定，並更新 user_setting_dict
    Read user settings from JSON file and update user_setting_dict
    """
    jeditor_logger.info("user_setting_file.py read_user_setting")

    # 設定檔路徑：當前工作目錄下的 `.jeditor/user_setting.json`
    # File path: `.jeditor/user_setting.json` under current working directory
    user_setting_file = Path(getcwd()) / ".jeditor" / "user_setting.json"

    # 如果檔案存在，就讀取並更新設定字典
    # If the file exists, read and update the settings dictionary
    if user_setting_file.exists() and user_setting_file.is_file():
        data = read_json(str(user_setting_file))
        if data is not None:
            user_setting_dict.update(data)



from pathlib import Path
from threading import Lock

# 匯入自訂例外與日誌工具
# Import custom exception and logging utility
from je_editor.utils.exception.exceptions import JEditorOpenFileException
from je_editor.utils.logging.loggin_instance import jeditor_logger

# 模組層級的鎖，確保多執行緒安全
# Module-level lock for thread safety
_file_read_lock = Lock()


def read_file(file_path: str) -> list[Path | str] | None:
    """
    功能說明 (Function Description):
    用來檢查檔案是否存在並嘗試開啟，讀取其內容。
    Used to check if a file exists and open it to read its content.

    :param file_path: 檔案完整路徑 / the full path of the file to read
    :return: [檔案路徑, 檔案內容] / [file path, file content]

    流程 (Logic):
    1. 嘗試鎖定執行緒 (避免多執行緒同時存取)
       Try to lock the thread (prevent concurrent access).
    2. 檢查檔案路徑是否為空，並確認檔案存在且為檔案。
       Check if file path is not empty, exists, and is a file.
    3. 若條件成立，嘗試以 UTF-8 編碼開啟檔案並讀取內容。
       If true, open the file with UTF-8 encoding and read content.
    4. 最後釋放鎖。
       Finally, release the lock.
    """

    # 記錄日誌，方便除錯與追蹤
    # Log the file path for debugging and tracking
    jeditor_logger.info(f"open_file.py read_file file_path: {file_path}")

    try:
        _file_read_lock.acquire()  # 嘗試鎖定資源 / Acquire the lock
        if file_path != "" and file_path is not None:  # 確認路徑不為空 / Ensure path is not empty
            file_path = Path(file_path)  # 轉換為 Path 物件 / Convert to Path object
            if file_path.exists() and file_path.is_file():  # 檢查檔案存在且為檔案 / Check file existence
                # 以唯讀模式開啟檔案 (UTF-8 編碼)
                # Open file in read-only mode with UTF-8 encoding
                with open(file_path, "r", encoding="utf-8") as open_read_file:
                    return [file_path, open_read_file.read()]  # 回傳檔案路徑與內容 / Return file path and content
    except (OSError, UnicodeDecodeError) as e:
        # 捕捉檔案 IO 與編碼例外
        # Catch file IO and encoding exceptions
        jeditor_logger.error(f"Failed to read file {file_path}: {e}")
        raise JEditorOpenFileException
    finally:
        # 確保鎖一定會被釋放
        # Ensure the lock is always released
        _file_read_lock.release()
